/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:RulesManagerModelLocator.as 2485 2008-09-30 14:14:35Z nicoulaj $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.rules.editor.model.presentation.editor
{
import flash.events.Event;
import flash.events.EventDispatcher;
import flash.sampler.*;

import mx.binding.utils.*;

import org.ow2.jasmine.monitoring.eos.rules.common.event.SynchronizedRulePackageEvent;
import org.ow2.jasmine.monitoring.eos.rules.common.model.domain.SynchronizedRulePackage;
import org.ow2.jasmine.monitoring.eos.rules.common.model.domain.SynchronizedRulePackageState;
import org.ow2.jasmine.monitoring.eos.rules.editor.event.*;
import org.ow2.jasmine.monitoring.eos.rules.editor.model.RuleEditorModelLocator;
    
/**
* Presentation model for the rule editor
* 
* @author Julien Nicoulaud
*/
[Bindable]
public class SynchronizedRulePackageEditorPM extends EventDispatcher
{
	
    // =========================================================================
    // Variables
    // =========================================================================
    
    /**
    * The edited rule package
    */
	public var editedSynchronizedRulePackage : SynchronizedRulePackage;
	
	/**
	* The editor tab label
	*/
	public var tabLabel : String;
	
	/**
	* The displayed message when the chosen rule package name is invalid
	*/
	public var nameErrorString : String = '';
    
    
    // =========================================================================
    // Constructors
    // =========================================================================
    
    /**
    * Build the Presentation Model for an engine dashboard view
    */
	public function SynchronizedRulePackageEditorPM(editedSynchronizedRulePackage:SynchronizedRulePackage)
	{
		// Initialize variables
		this.editedSynchronizedRulePackage = editedSynchronizedRulePackage;
		
		// Refresh values for the first time
		updateTabLabel();
		
        // Watch some model variables changes
        ChangeWatcher.watch(this.editedSynchronizedRulePackage,"state",updateTabLabel);
        ChangeWatcher.watch(this.editedSynchronizedRulePackage,"state",handleStateChange);
        ChangeWatcher.watch(this.editedSynchronizedRulePackage.rulePackage,"displayName",updateTabLabel);
        ChangeWatcher.watch(this.editedSynchronizedRulePackage.rulePackage,"systemName",checkNameAvailability);
        ChangeWatcher.watch(this.editedSynchronizedRulePackage.rulePackage,"nameIsValid",updateErrorString);
        ChangeWatcher.watch(this.editedSynchronizedRulePackage.rulePackage,"nameIsAvailable",updateErrorString);
        
	}
	
    // =========================================================================
    // Public methods
    // =========================================================================	        
    
    /**
    * Test the edited rule package compilation
    */
    public function compileRulePackage(e:Event=null):void
    {
        new SynchronizedRulePackageEvent(SynchronizedRulePackageEvent.COMPILE_RULE_PACKAGE + RuleEditorModelLocator.getInstance().ruleEditorModel.componentID,
                                         editedSynchronizedRulePackage).dispatch();
    }
    
    /**
    * Test the edited rule package name is available
    */
    public function checkNameAvailability(e:Event=null):void
    {
        new SynchronizedRulePackageEvent(SynchronizedRulePackageEvent.CHECK_NAME_AVAILABILITY + RuleEditorModelLocator.getInstance().ruleEditorModel.componentID,
                                         editedSynchronizedRulePackage).dispatch();
    }
    
    
    // =========================================================================
    // Private methods
    // =========================================================================
    
    /**
    * Update the displayed message when the chosen rule package name is invalid
    */
    protected function updateErrorString(e:Event=null):void
    {
        if (!editedSynchronizedRulePackage.rulePackage.nameIsAvailable) {
            nameErrorString = 'The name you entered is already taken by another rule package';
        } else if (!editedSynchronizedRulePackage.rulePackage.nameIsValid) {
            nameErrorString = 'The name you entered is invalid. Special characters are forbidden.';
        } else {
            nameErrorString = '';
        }
    }
    
    /**
    * Update the label displayed on the editor tab
    */
    protected function updateTabLabel(e:Event=null):void
    {
        this.tabLabel = editedSynchronizedRulePackage.rulePackage.displayName;
        if(editedSynchronizedRulePackage.state == SynchronizedRulePackageState.MODIFIED) {
            this.tabLabel += " *";
        }
        if(editedSynchronizedRulePackage.state == SynchronizedRulePackageState.OBSOLETE ||
           editedSynchronizedRulePackage.state == SynchronizedRulePackageState.OUT_OF_SYNC) {
            this.tabLabel += " (!)";
        }
    }
    
    /**
    * Handle the synchronized rule package state change
    */
    protected function handleStateChange(e:Event=null):void
    {
        if(editedSynchronizedRulePackage.state == SynchronizedRulePackageState.OBSOLETE ||
           editedSynchronizedRulePackage.state == SynchronizedRulePackageState.OUT_OF_SYNC) {
            new SynchronizedRulePackageEvent(SynchronizedRulePackageEvent.UPDATE_RULE_PACKAGE_METADATA + RuleEditorModelLocator.getInstance().ruleEditorModel.componentID,
                                             editedSynchronizedRulePackage).dispatch();
        }
    }

}
}
